from pathlib import Path
from typing import Literal

from pydantic import BaseModel


VoiceIdMap = {
    "kuon": "B8gJV1IhpuegLxdpXFOE",
    "Ishibashi": "Mv8AjrYZCBkdsmDHNwcB",
    "Satoshi": "V3XiX7JWJpn959SS60pv",
    "Morioki": "8EkOjt4xTPGMclNlh1pk"
}


VoiceNameTypes = Literal["kuon", "Ishibashi", "Satoshi", "Morioki"]


class TextToAudioRequest(BaseModel):
    text: str
    voice_name: VoiceNameTypes


class TextToAudioResponse(BaseModel):
    audio: bytes

    def save_mp3(self, path: str | Path):
        if not isinstance(path, Path):
            path = Path(path)
        if not path.suffix == ".mp3":
            raise ValueError("path must end with .mp3")
        with open(path, "wb") as f:
            f.write(self.audio)




import io
import wave
from pathlib import Path
from typing import Literal

from pydantic import BaseModel
from pydub import AudioSegment


VoiceNameTypes = Literal[
    "Zephyr",  # 明るい
    "Puck",  # アップビート
    "Charon",  # 情報提供
    "Kore",  # 会社
    "Fenrir",  # 興奮しやすい
    "Leda",  # 若々しい
    "Orus",  # 会社
    "Aoede",  # Breezy
    "Callirhoe",  # 気楽な
    "Autonoe",  # 明るい
    "Enceladus",  # 息づかい
    "Iapetus",  # クリア
    "Umbriel",  # 気楽な
    "Algieba",  # スムーズ
    "Despina",  # スムーズ
    "Erinome",  # クリア
    "Algenib",  # 砂利
    "Rasalgethi",  # 情報に富んでいる
    "Laomedeia",  # アップビート
    "Achernar",  # ソフト
    "Alnilam",  # 確実
    "Schedar",  # Even
    "Gacrux",  # 成人向け
    "Pulcherrima",  # 前方
    "Achird",  # フレンドリー
    "Zubenelgenubi",  # カジュアル
    "Vindemiatrix",  # 優しい
    "Sadachbia",  # 活発
    "Sadaltager",  # 知識豊富
    "Sulafar",  # 温かい
]


class TextToAudioRequest(BaseModel):
    text: str
    instructions: str
    voice_name: VoiceNameTypes = "Kore"


class TextToAudioResponse(BaseModel):
    audio: bytes  # wav

    @classmethod
    def from_bytes(cls, audio: bytes) -> "TextToAudioResponse":
        buf = io.BytesIO()
        with wave.open(buf, "wb") as wf:
            wf.setnchannels(1)
            wf.setsampwidth(2)
            wf.setframerate(24000)
            wf.writeframes(audio)
        return cls(audio=buf.getvalue())

    def save_wav(self, path: str | Path):
        if not isinstance(path, Path):
            path = Path(path)
        if not path.suffix == ".wav":
            raise ValueError("path must end with .wav")
        with open(path, "wb") as f:
            f.write(self.audio)

    def save_mp3(self, path: str | Path):
        if not isinstance(path, Path):
            path = Path(path)
        if not path.suffix == ".mp3":
            raise ValueError("path must end with .mp3")
        wav_io = io.BytesIO(self.audio)
        audio = AudioSegment.from_file(wav_io, format="wav")
        mp3_io = io.BytesIO()
        audio.export(mp3_io, format="mp3", bitrate="192k")
        with open(path, "wb") as f:
            f.write(mp3_io.getvalue())


class SpeakerTextToAudioRequest(BaseModel):
    speaker_name: str
    text: str
    voice_name: VoiceNameTypes = "Kore"


class MultiSpeakerTextToAudioRequest(BaseModel):
    speakers: list[SpeakerTextToAudioRequest]
    instructions: str



from typing import Any, Literal
from pathlib import Path

from pydantic import BaseModel, ConfigDict, Field, field_validator


MinimaxModels = Literal["speech-02-turbo", "speech-02-hd", "speech-01-turbo", "speech-01-hd"]
MinimaxVoiceIds = Literal[
    "Wise_Woman",
    "Friendly_Person",
    "Inspirational_girl",
    "Deep_Voice_Man",
    "Calm_Woman",
    "Casual_Guy",
    "Lively_Girl",
    "Patient_Man",
    "Young_Knight",
    "Determined_Man",
    "Lovely_Girl",
    "Decent_Boy",
    "Imposing_Manner",
    "Elegant_Man",
    "Abbess",
    "Sweet_Girl_2",
    "Exuberant_Girl",
    "Japanese_GentleButler",
    "Japanese_LoyalKnight",
]

MiniMaxEmotions = Literal[
    "happy",
    "sad",
    "angry",
    "fearful",
    "disgusted",
    "surprised",
    "neutral",
]


class BaseResp(BaseModel):
    status_code: int
    status_msg: str


class VoiceSetting(BaseModel):
    voice_id: MinimaxVoiceIds | str = "Japanese_LoyalKnight"
    speed: float = Field(1.0, ge=0.5, le=2.0)
    vol: float = Field(1.0, ge=0, le=2.0)
    pitch: int = Field(0, ge=-12, le=12)
    emotion: MiniMaxEmotions | None = None
    english_normalization: bool = False


class AudioSetting(BaseModel):
    audio_sample_rate: Literal[24000, 32000, 48000] = 32000
    bitrate: Literal[32000, 64000, 128000] = 128000
    format: Literal["mp3", "pcm", "flac"] = "mp3"
    channel: Literal[1, 2] = 1


class TextToAudioRequest(BaseModel):
    """
    T2A v2 Request Model
    """
    text: str
    model: MinimaxModels = "speech-02-turbo"
    stream: bool = False
    voice_setting: VoiceSetting = VoiceSetting()
    audio_setting: AudioSetting = AudioSetting()
    pronunciation_dict: dict[str, str] | None = None

    model_config = ConfigDict(extra="allow")

    @field_validator("text")
    def _trim_length(cls, v):
        if len(v) > 5000:
            raise ValueError("text length exceeds 5000 characters (T2A v2 hard limit)")
        return v


class AudioData(BaseModel):
    audio: str
    status: int

    def to_bytes(self) -> bytes:
        return bytes.fromhex(self.audio)

    def save_mp3(self, path: str | Path):
        if not isinstance(path, Path):
            path = Path(path)
        if not path.suffix == ".mp3":
            raise ValueError("path must end with .mp3")
        with open(path, "wb") as f:
            f.write(self.to_bytes())


class TextToAudioResponse(BaseModel):
    data: AudioData | None = None
    trace_id: str | None = None
    subtitle_file: str | None = None
    base_resp: BaseResp
    extra_info: dict[str, Any] | None = None



from google import genai
from google.genai import types

from .models import MultiSpeakerTextToAudioRequest, TextToAudioRequest, TextToAudioResponse


class GoogleTTSClient:
    def __init__(self, model: str = "gemini-2.5-flash-preview-tts"):
        self.client = genai.Client()
        self.model = model

    def text_to_audio(self, req: TextToAudioRequest) -> TextToAudioResponse:
        response = self.client.models.generate_content(
            model=self.model,
            contents=f"{req.instructions}: {req.text}",
            config=types.GenerateContentConfig(
                response_modalities=["AUDIO"],
                speech_config=types.SpeechConfig(
                    voice_config=types.VoiceConfig(
                        prebuilt_voice_config=types.PrebuiltVoiceConfig(
                            voice_name=req.voice_name,
                        )
                    ),
                ),
            ),
        )
        data = response.candidates[0].content.parts[0].inline_data.data
        return TextToAudioResponse.from_bytes(data)

    def multi_speaker_text_to_audio(self, req: MultiSpeakerTextToAudioRequest) -> TextToAudioResponse:
        prompt_text = "\n".join([
            f"{speaker.speaker_name}: {speaker.text}"
            for speaker in req.speakers
        ])
        speakers_set = set([(speaker.speaker_name, speaker.voice_name) for speaker in req.speakers])
        speech_config=types.SpeechConfig(
           multi_speaker_voice_config=types.MultiSpeakerVoiceConfig(
              speaker_voice_configs=[
                 types.SpeakerVoiceConfig(
                    speaker=speaker[0],
                    voice_config=types.VoiceConfig(
                       prebuilt_voice_config=types.PrebuiltVoiceConfig(
                          voice_name=speaker[1],
                       )
                    )
                 )
                 for speaker in speakers_set
              ]
           )
        )
        response = self.client.models.generate_content(
            model=self.model,
            contents=f"{req.instructions}:\n{prompt_text}",
            config=types.GenerateContentConfig(
                response_modalities=["AUDIO"],
                speech_config=speech_config,
            ),
        )
        data = response.candidates[0].content.parts[0].inline_data.data
        return TextToAudioResponse.from_bytes(data)



from pathlib import Path
from typing import Any

from pydantic import BaseModel


class TextToAudioRequest(BaseModel):
    text: str
    instructions: str
    voice_name: str
    speed: float = 1.0


class TextToAudioResponse(BaseModel):
    audio_stream: Any

    def save_mp3(self, path: str | Path):
        if not isinstance(path, Path):
            path = Path(path)
        if not path.suffix == ".mp3":
            raise ValueError("path must end with .mp3")
        self.audio_stream.stream_to_file(path)



import os

from elevenlabs import ElevenLabs

from .models import TextToAudioRequest, TextToAudioResponse, VoiceIdMap


class ElevenLabsClient:
    def __init__(self, model_id: str = "eleven_v3"):
        self.client = ElevenLabs(api_key=os.getenv("ELEVENLABS_API_KEY"),)
        self.model_id = model_id

    def text_to_audio(self, request: TextToAudioRequest) -> TextToAudioResponse:
        audio = self.client.text_to_speech.convert(
            text=request.text,
            voice_id=VoiceIdMap[request.voice_name],
            model_id=self.model_id,
            output_format="mp3_44100_128",
        )
        audio = b"".join(audio)
        return TextToAudioResponse(audio=audio)



from dotenv import load_dotenv

from tts_clients.minmax.client import MiniMaxT2AClient
from tts_clients.minmax.models import TextToAudioRequest, VoiceSetting


load_dotenv()


client = MiniMaxT2AClient()
r = client.text_to_audio(TextToAudioRequest(text="こんにちは！", voice_setting=VoiceSetting(emotion="angry")))
r.data.save_mp3("test.mp3")



from dotenv import load_dotenv

from tts_clients.google.client import GoogleTTSClient
from tts_clients.google.models import MultiSpeakerTextToAudioRequest, SpeakerTextToAudioRequest


load_dotenv()

client = GoogleTTSClient()
r = client.multi_speaker_text_to_audio(MultiSpeakerTextToAudioRequest(
    speakers=[
        SpeakerTextToAudioRequest(speaker_name="speaker1", text="こんにちは！", voice_name="Kore"),
        SpeakerTextToAudioRequest(speaker_name="speaker2", text="こんばんは！", voice_name="Orus"),
        SpeakerTextToAudioRequest(speaker_name="speaker1", text="今日は晴れですね", voice_name="Kore"),
    ],
    instructions="Say cheerfully",
))
r.save_mp3("test.mp3")



import os
from typing import Any

import httpx
from pydantic import BaseModel

from .models import TextToAudioRequest, TextToAudioResponse


_DEFAULT_HOST = "https://api.minimaxi.chat/v1"


def _default_headers(api_key: str) -> dict[str, str]:
    return {"Authorization": f"Bearer {api_key}", "Content-Type": "application/json"}


class MiniMaxT2AClient:
    """
    MiniMax T2A v2 Client
    """
    def __init__(
        self,
        api_key: str | None = None,
        group_id: str | None = None,
        host: str = _DEFAULT_HOST,
        *,
        httpx_client: httpx.Client | None = None,
        timeout: float = 60.0,
    ) -> None:
        self._api_key = api_key or os.environ.get("MINIMAX_API_KEY")
        self._group_id = group_id or os.environ.get("MINIMAX_GROUP_ID")
        self._base_url = host.rstrip("/")
        self._timeout = timeout
        self._client = httpx_client

    def text_to_audio(self, req: TextToAudioRequest) -> TextToAudioResponse:
        return self._run_sync(
            "t2a_v2",
            req.model_dump(exclude_none=True),
            TextToAudioResponse,
        )

    def _endpoint(self, path: str) -> str:
        return f"{self._base_url}/{path}?GroupId={self._group_id}"

    def _run_sync(self, path: str, body: dict[str, Any], model: type[BaseModel]):
        client = self._client or httpx.Client(timeout=self._timeout)
        try:
            r = client.post(
                self._endpoint(path),
                headers=_default_headers(self._api_key),
                json=body,
            )
            r.raise_for_status()
            return model.model_validate(r.json())
        finally:
            if not self._client:
                client.close()



from openai import OpenAI

from .models import TextToAudioRequest, TextToAudioResponse


class OpenAITTSClient:
    def __init__(self):
        self.client = OpenAI()

    def text_to_audio(self, req: TextToAudioRequest) -> TextToAudioResponse:
        response = self.client.audio.speech.create(
            model="gpt-4o-mini-tts",
            voice=req.voice_name,
            input=req.text,
            instructions=req.instructions,
            speed=req.speed,
        )
        return TextToAudioResponse(audio_stream=response)



from dotenv import load_dotenv

from tts_clients.elevenlabs.client import ElevenLabsClient
from tts_clients.elevenlabs.models import TextToAudioRequest


load_dotenv()

client = ElevenLabsClient()
request = TextToAudioRequest(text="こんにちは！", voice_name="kuon")
response = client.text_to_audio(request)
response.save_mp3("test.mp3")
